import { CID } from "@lumeweb/libs5";
import { blake3 } from "@noble/hashes/blake3";
import { equalBytes } from "@noble/ciphers/utils";
import { defer } from "@lumeweb/libkernel/module";

document.title = "Hosted Lume Kernel Worker";
let header = document.createElement("h1");
header.textContent =
  "Something went wrong! You should not be visiting this page, this page should only be accessed via an invisible iframe.";
document.body.appendChild(header);

const initDefer = defer<Uint8Array>();

let worker: Worker;

function boot() {
  let workerCID = window.location.hostname.split(".")[0];

  let cid = CID.decode(workerCID);

  initDefer.promise.then((module) => {
    if (!equalBytes(blake3(module), cid.hash.hashBytes)) {
      sendError("invalid module");
      return;
    }

    const moduleUrl = URL.createObjectURL(new Blob([module]));

    worker = new Worker(moduleUrl);

    worker.onmessage = function (event: MessageEvent) {
      sendMessage("workerMessage", event.data);
    };
    worker.onerror = function (event: ErrorEvent) {
      sendError(event.message, event.error);
    };

    sendMessage("workerInited", {});
  });
}

function sendMessage(method: string, data: any) {
  window.parent.postMessage(
    {
      method,
      data,
    },
    "https://kernel.lumeweb.com",
  );
}

function sendError(message: string, error = null) {
  sendMessage("workerError", {
    data: {
      message,
      error,
    },
  });
}

onmessage = function (event: MessageEvent) {
  if (window === event.source) {
    return;
  }
  if (event.origin !== "https://kernel.lumeweb.com") {
    return;
  }

  if (event.data.method === "workerInit") {
    initDefer.resolve(event.data.module);
    return;
  }

  worker?.postMessage(event.data);
};

boot();
